using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Net;
using System.IO;

namespace RusToEng
{
    /// <summary>
    /// Author		: zephyrrr
    /// Description : Translate by web translate engine
    /// </summary>
    public class TranslateByWeb
    {
        public enum TranslateEngine
        {
            Google,
            BabelFish,
            Paralink
        }

        public enum TranslateLanguage
        {
            RussianToEnglish
        }

        private static string Translate(TranslateEngine engine, TranslateLanguage language, string srcString)
        {
            srcString = srcString.Trim();

            //CookieContainer cookieContainer = new CookieContainer();
            //Uri bbsRoot = new Uri("http://translate.google.com");
            //cookieContainer.Add(bbsRoot, new Cookie("PREF=ID", "b46073674e7d"));   

            HttpWebRequest request;

            string postData;
            switch (engine)
            {
                case TranslateEngine.Google:
                    request = (HttpWebRequest)WebRequest.Create("http://translate.google.com/translate_t");
                    request.Referer = "http://www.google.com/translate_t";
                    switch (language)
                    {
                        case TranslateLanguage.RussianToEnglish:
                            postData = "text=" + srcString + "&langpair=" + "ru%7Cen" + "&hl=en&newwindow=1&ie=UTF-8&oe=UTF-8";
                            break;
                        default:
                            throw new NotSupportedException("not supported translate language");
                    }
                    break;
                case TranslateEngine.BabelFish:
                    request = (HttpWebRequest)WebRequest.Create("http://babelfish.altavista.com/tr");
                    request.Referer = "http://babelfish.altavista.com/tr";
                    switch (language)
                    {
                        case TranslateLanguage.RussianToEnglish:
                            postData = "doit=done&intl=1&tt=urltext&trtext=" + srcString + "&lp=" + "ru_en" + "&btnTrTxt=Translate";
                            break;
                        default:
                            throw new NotSupportedException("not supported translate language");
                    }
                    break;
                case TranslateEngine.Paralink:
                    request = (HttpWebRequest)WebRequest.Create("http://translation2.paralink.com/newbot.asp");
                    switch (language)
                    {
                        case TranslateLanguage.RussianToEnglish:
                            postData = "status=translate&directions=" + "ru%2Fen" + "&moredirs=More+Languages...&source=" + srcString + "&accent1=%C3%A0&accent2=%C3%A8&accent3=%C3%AC&accent4=%C3%B2&accent5=%C3%B9&accent6=misc";
                            break;
                        default:
                            throw new NotSupportedException("not supported translate language");
                    }
                    break;
                default:
                    throw new NotSupportedException("not supported engine");
            }
            
            request.Method = "POST";
            request.Accept = "image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, application/x-shockwave-flash, application/msword, application/vnd.ms-excel, application/vnd.ms-powerpoint, */*";
            request.UserAgent = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Maxthon; .NET CLR 2.0.50727)";
            request.ContentType = "application/x-www-form-urlencoded";
            
            //request.CookieContainer = cookieContainer;
            //request.Credentials = credential;
            //request.KeepAlive = true;
            //request.Headers.Add("Accept-Language", "en-us");
            //request.Headers.Add("UA-CPU", "x86");
            //request.Headers.Add("Accept-Encoding", "gzip, deflate");
            //request.Headers.Add("Cache-Control", "no-cache");
            //request.Proxy = new WebProxy("127.0.0.1", 8088);


            byte[] byteArray = Encoding.UTF8.GetBytes(postData);
            request.ContentLength = byteArray.Length;
            Stream dataStream = request.GetRequestStream();
            dataStream.Write(byteArray, 0, byteArray.Length);
            dataStream.Close();
            HttpWebResponse response = (HttpWebResponse)request.GetResponse();

            //response.Cookies = cookieContainer.GetCookies(request.RequestUri);
            //if (response.Headers["SetCookies"] != null && response.Headers["SetCookies"] != string.Empty)
            //{
            //    cookieContainer.SetCookies(bbsRoot, response.Headers["SetCookies"]);
            //}   

            dataStream = response.GetResponseStream();
            StreamReader reader = new StreamReader(dataStream);
            string responseFromServer = reader.ReadToEnd();
            reader.Close();
            dataStream.Close();
            response.Close();

            Regex resultPattern;
            MatchCollection mc;
            
            switch (engine)
            {
                case TranslateEngine.Google:
                    resultPattern = new Regex(@"<td id=result_cell><div id=result_box dir=ltr>(.*?)</div></td>", RegexOptions.IgnoreCase);
                    mc = resultPattern.Matches(responseFromServer);
                    if (mc.Count > 0)
                    {
                        return mc[0].Groups[1].Value;
                    }
                    else
                    {
                        throw new Exception("u are baned by google. " + responseFromServer);
                    }
                case TranslateEngine.BabelFish:
                    resultPattern = new Regex(@"<div style=padding:10px;>((?:.|\n)*?)</div>", RegexOptions.IgnoreCase);
                    mc = resultPattern.Matches(responseFromServer);
                    if (mc.Count != 1 || mc[0].Groups.Count != 2)
                    {
                        throw new Exception("u are baned by BabelFish. " + responseFromServer);
                    }
                    return mc[0].Groups[1].Value;
                default:
                    throw new NotSupportedException("not supported engine"); 
            }
        }

        private static bool IsRussianInString(byte[] rusUnicode)
        {
            // 0x0400 - 0x052F
            for (int i = 0; i < rusUnicode.Length / 2; ++i)
            {
                int n = rusUnicode[2 * i] + (rusUnicode[2 * i + 1] << 8);
                if (n >= 0x0400 && n <= 0x052F)
                {
                    return true;
                }
            }
            return false;
        }

        // Create two different encodings.
        private static Encoding rus = Encoding.GetEncoding(1251);// Encoding.ASCII;
        private static Encoding unicode = Encoding.Unicode;
        private static Encoding ascii = Encoding.ASCII;
        public static void TranslateFile(TranslateEngine engine, TranslateLanguage language, string fileSrc, string fileDest, bool translateAll)
        {
            if (File.Exists(fileDest))
            {
                FileInfo f = new FileInfo(fileDest);
                if (f.Length != 0)
                {
                    return;
                }
            }

            string directory = null;
            int dirIndex = fileDest.LastIndexOf('\\');
            if (dirIndex != -1)
            {
                directory = fileDest.Substring(0, dirIndex);
            }
            if (!string.IsNullOrEmpty(directory))
            {
                DirectoryInfo dir = new DirectoryInfo(directory);
                if (!dir.Exists)
                {
                    dir.Create();
                }
            }

            StreamReader sr;
            StreamWriter sw;
            Encoding srcEncoding;
            switch (language)
            {
                case TranslateLanguage.RussianToEnglish:
                    sr = new StreamReader(fileSrc, rus, true);
                    sw = new StreamWriter(fileDest, false, ascii);
                    srcEncoding = rus;
                    break;
                default:
                    throw new NotSupportedException("not supported engine");
            }

            string srcString;

            //bool comment1 = false;  // //
            //bool comment2 = false;  // /* */
            //bool comment3 = false;  // ""
            string line;
            while ((line = sr.ReadLine()) != null)
            {
                Console.Write("#");

                srcString = line;
                byte[] srcBytes = srcEncoding.GetBytes(srcString);
                byte[] unicodeBytes = Encoding.Convert(rus, unicode, srcBytes);
                char[] unicodeChars = new char[unicode.GetCharCount(unicodeBytes, 0, unicodeBytes.Length)];
                unicode.GetChars(unicodeBytes, 0, unicodeBytes.Length, unicodeChars, 0);
                string unicodeString = new string(unicodeChars);

                string translatedString = "";
                bool containTranslate = false;
                switch (language)
                {
                    case TranslateLanguage.RussianToEnglish:
                        containTranslate = IsRussianInString(unicodeBytes);
                        break;
                    default:
                        throw new NotSupportedException("not supported engine");
                }
                if (containTranslate || translateAll)
                {
                    System.Threading.Thread.Sleep(1000);

                    bool success = false;
                    int count = 0;
                    while (!success)
                    {
                        try
                        {
                            count++;
                            translatedString = Translate(engine, language, unicodeString);
                            success = true;
                        }
                        catch (Exception)
                        {
                            Console.Write("e");
                            System.Threading.Thread.Sleep(count * 20000);
                            if (count > 50)
                            {
                                Console.Write("E");
                                translatedString = unicodeString;
                                success = true;
                            }
                        }
                    }

                    translatedString = translatedString.Trim().Replace("&lt;", "").Replace("&gt;", "").Replace("&quot;", "").Replace("\r", "").Replace("\n", "");
                    translatedString = translatedString.Replace("/ /", "//").Replace("/ *", "/*").
                            Replace("* /", "*/");
                    translatedString = translatedString.Replace("/ ", "//");
                }
                else
                {
                    translatedString = unicodeString;
                }

                sw.WriteLine(translatedString);
                //sw2.WriteLine(unicodeString);
            }

            sr.Close();
            sw.Close();
            //sw2.Close();
        }
    }
}
